/* arch/arm/plat-s5pc1xx/include/plat/regs-clock.h
 *
 * Copyright 2008 Samsung Electronics
 * Copyright 2008 Simtec Electronics
 *	Ben Dooks <ben@simtec.co.uk>
 *	http://armlinux.simtec.co.uk/
 *
 * S5PC1XX clock register definitions
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

#ifndef __PLAT_REGS_CLOCK_H
#define __PLAT_REGS_CLOCK_H __FILE__

#define S5P_CLKREG(x)		(S3C_VA_SYS + (x))

#define S5P_IECREG(x)		(S5PC1XX_VA_IEC + (x))
#define S5P_APCREG(x)		(S5PC1XX_VA_APC + (x))

#if defined(CONFIG_CPU_S5PC100)
#define S5P_APLL_LOCK		S5P_CLKREG(0x00)
#define S5P_MPLL_LOCK		S5P_CLKREG(0x04)
#define S5P_EPLL_LOCK		S5P_CLKREG(0x08)
#define S5P_HPLL_LOCK		S5P_CLKREG(0x0C)

#define S5P_APLL_CON		S5P_CLKREG(0x100)
#define S5P_MPLL_CON		S5P_CLKREG(0x104)
#define S5P_EPLL_CON		S5P_CLKREG(0x108)
#define S5P_HPLL_CON		S5P_CLKREG(0x10C)

#define S5P_CLK_SRC0		S5P_CLKREG(0x200)
#define S5P_CLK_SRC1		S5P_CLKREG(0x204)
#define S5P_CLK_SRC2		S5P_CLKREG(0x208)
#define S5P_CLK_SRC3		S5P_CLKREG(0x20C)

#define S5P_CLK_DIV0		S5P_CLKREG(0x300)
#define S5P_CLK_DIV1		S5P_CLKREG(0x304)
#define S5P_CLK_DIV2		S5P_CLKREG(0x308)
#define S5P_CLK_DIV3		S5P_CLKREG(0x30C)
#define S5P_CLK_DIV4		S5P_CLKREG(0x310)

#define S5P_CLK_OUT		S5P_CLKREG(0x400)

#define S5P_CLKGATE_D00		S5P_CLKREG(0x500)
#define S5P_CLKGATE_D01		S5P_CLKREG(0x504)
#define S5P_CLKGATE_D02		S5P_CLKREG(0x508)

#define S5P_CLKGATE_D10		S5P_CLKREG(0x520)
#define S5P_CLKGATE_D11		S5P_CLKREG(0x524)
#define S5P_CLKGATE_D12		S5P_CLKREG(0x528)
#define S5P_CLKGATE_D13		S5P_CLKREG(0x52C)
#define S5P_CLKGATE_D14		S5P_CLKREG(0x530)
#define S5P_CLKGATE_D15		S5P_CLKREG(0x534)

#define S5P_CLKGATE_D20		S5P_CLKREG(0x540)

#define S5P_SCLKGATE0		S5P_CLKREG(0x560)
#define S5P_SCLKGATE1		S5P_CLKREG(0x564)

#define S5P_OTHERS              S5P_CLKREG(0x8200)

#define S5P_EPLL_EN     (1<<31)
#define S5P_EPLL_MASK   0xffffffff 
#define S5P_EPLLVAL(_m,_p,_s)   ((_m) << 16 | ((_p) << 8) | ((_s)))

/* CLKSRC0 */
#define S5P_CLKSRC0_APLL_MASK		(0x1<<0)
#define S5P_CLKSRC0_APLL_SHIFT		(0)
#define S5P_CLKSRC0_MPLL_MASK		(0x1<<4)
#define S5P_CLKSRC0_MPLL_SHIFT		(4)
#define S5P_CLKSRC0_EPLL_MASK		(0x1<<8)
#define S5P_CLKSRC0_EPLL_SHIFT		(8)
#define S5P_CLKSRC0_HPLL_MASK		(0x1<<12)
#define S5P_CLKSRC0_HPLL_SHIFT		(12)
#define S5P_CLKSRC0_AMMUX_MASK		(0x1<<16)
#define S5P_CLKSRC0_AMMUX_SHIFT		(16)
#define S5P_CLKSRC0_HREF_MASK		(0x1<<20)
#define S5P_CLKSRC0_HREF_SHIFT		(20)
#define S5P_CLKSRC0_ONENAND_MASK	(0x1<<24)
#define S5P_CLKSRC0_ONENAND_SHIFT	(24)

/* CLKSRC1 */
#define S5P_CLKSRC1_UART_MASK		(0x1<<0)
#define S5P_CLKSRC1_UART_SHIFT		(0)
#define S5P_CLKSRC1_SPI0_MASK		(0x3<<4)
#define S5P_CLKSRC1_SPI0_SHIFT		(4)
#define S5P_CLKSRC1_SPI1_MASK		(0x3<<8)
#define S5P_CLKSRC1_SPI1_SHIFT		(8)
#define S5P_CLKSRC1_SPI2_MASK		(0x3<<12)
#define S5P_CLKSRC1_SPI2_SHIFT		(12)
#define S5P_CLKSRC1_IRDA_MASK		(0x3<<16)
#define S5P_CLKSRC1_IRDA_SHIFT		(16)
#define S5P_CLKSRC1_UHOST_MASK		(0x3<<20)
#define S5P_CLKSRC1_UHOST_SHIFT		(20)
#define S5P_CLKSRC1_CLK48M_MASK		(0x1<<24)
#define S5P_CLKSRC1_CLK48M_SHIFT	(24)

/* CLKSRC2 */
#define S5P_CLKSRC2_MMC0_MASK		(0x3<<0)
#define S5P_CLKSRC2_MMC0_SHIFT		(0)
#define S5P_CLKSRC2_MMC1_MASK		(0x3<<4)
#define S5P_CLKSRC2_MMC1_SHIFT		(4)
#define S5P_CLKSRC2_MMC2_MASK		(0x3<<8)
#define S5P_CLKSRC2_MMC2_SHIFT		(8)
#define S5P_CLKSRC2_LCD_MASK		(0x3<<12)
#define S5P_CLKSRC2_LCD_SHIFT		(12)
#define S5P_CLKSRC2_FIMC0_MASK		(0x3<<16)
#define S5P_CLKSRC2_FIMC0_SHIFT		(16)
#define S5P_CLKSRC2_FIMC1_MASK		(0x3<<20)
#define S5P_CLKSRC2_FIMC1_SHIFT		(20)
#define S5P_CLKSRC2_FIMC2_MASK		(0x3<<24)
#define S5P_CLKSRC2_FIMC2_SHIFT		(24)
#define S5P_CLKSRC2_MIXER_MASK		(0x3<<28)
#define S5P_CLKSRC2_MIXER_SHIFT		(28)

/* CLKSRC3 */
#define S5P_CLKSRC3_PWI_MASK		(0x3<<0)
#define S5P_CLKSRC3_PWI_SHIFT		(0)
#define S5P_CLKSRC3_HCLKD2_MASK		(0x1<<4)
#define S5P_CLKSRC3_HCLKD2_SHIFT	(4)
#define S5P_CLKSRC3_I2SD2_MASK		(0x3<<8)
#define S5P_CLKSRC3_I2SD2_SHIFT		(8)
#define S5P_CLKSRC3_AUDIO0_MASK		(0x7<<12)
#define S5P_CLKSRC3_AUDIO0_SHIFT	(12)
#define S5P_CLKSRC3_AUDIO1_MASK		(0x7<<16)
#define S5P_CLKSRC3_AUDIO1_SHIFT	(16)
#define S5P_CLKSRC3_AUDIO2_MASK		(0x7<<20)
#define S5P_CLKSRC3_AUDIO2_SHIFT	(20)
#define S5P_CLKSRC3_SPDIF_MASK		(0x3<<24)
#define S5P_CLKSRC3_SPDIF_SHIFT		(24)


/* CLKDIV0 */
#define S5P_CLKDIV0_APLL_MASK		(0x1<<0)
#define S5P_CLKDIV0_APLL_SHIFT		(0)
#define S5P_CLKDIV0_ARM_MASK		(0x7<<4)
#define S5P_CLKDIV0_ARM_SHIFT		(4)
#define S5P_CLKDIV0_D0_MASK		(0x7<<8)
#define S5P_CLKDIV0_D0_SHIFT		(8)
#define S5P_CLKDIV0_PCLKD0_MASK		(0x7<<12)
#define S5P_CLKDIV0_PCLKD0_SHIFT	(12)
#define S5P_CLKDIV0_SECSS_MASK		(0x7<<16)
#define S5P_CLKDIV0_SECSS_SHIFT		(16)

/* CLKDIV1 */
#define S5P_CLKDIV1_AM_MASK		(0x7<<0)
#define S5P_CLKDIV1_AM_SHIFT		(0)
#define S5P_CLKDIV1_MPLL_MASK		(0x3<<4)
#define S5P_CLKDIV1_MPLL_SHIFT		(4)
#define S5P_CLKDIV1_MPLL2_MASK		(0x1<<8)
#define S5P_CLKDIV1_MPLL2_SHIFT		(8)
#define S5P_CLKDIV1_D1_MASK		(0x7<<12)
#define S5P_CLKDIV1_D1_SHIFT		(12)
#define S5P_CLKDIV1_PCLKD1_MASK		(0x7<<16)
#define S5P_CLKDIV1_PCLKD1_SHIFT	(16)
#define S5P_CLKDIV1_ONENAND_MASK	(0x3<<20)
#define S5P_CLKDIV1_ONENAND_SHIFT	(20)
#define S5P_CLKDIV1_CAM_MASK		(0x1F<<24)
#define S5P_CLKDIV1_CAM_SHIFT		(24)

/* CLKDIV2 */
#define S5P_CLKDIV2_UART_MASK		(0x7<<0)
#define S5P_CLKDIV2_UART_SHIFT		(0)
#define S5P_CLKDIV2_SPI0_MASK		(0xf<<4)
#define S5P_CLKDIV2_SPI0_SHIFT		(4)
#define S5P_CLKDIV2_SPI1_MASK		(0xf<<8)
#define S5P_CLKDIV2_SPI1_SHIFT		(8)
#define S5P_CLKDIV2_SPI2_MASK		(0xf<<12)
#define S5P_CLKDIV2_SPI2_SHIFT		(12)
#define S5P_CLKDIV2_IRDA_MASK		(0xf<<16)
#define S5P_CLKDIV2_IRDA_SHIFT		(16)
#define S5P_CLKDIV2_UHOST_MASK		(0xf<<20)
#define S5P_CLKDIV2_UHOST_SHIFT		(20)

/* CLKDIV3 */
#define S5P_CLKDIV3_MMC0_MASK		(0xf<<0)
#define S5P_CLKDIV3_MMC0_SHIFT		(0)
#define S5P_CLKDIV3_MMC1_MASK		(0xf<<4)
#define S5P_CLKDIV3_MMC1_SHIFT		(4)
#define S5P_CLKDIV3_MMC2_MASK		(0xf<<8)
#define S5P_CLKDIV3_MMC2_SHIFT		(8)
#define S5P_CLKDIV3_LCD_MASK		(0xf<<12)
#define S5P_CLKDIV3_LCD_SHIFT		(12)
#define S5P_CLKDIV3_FIMC0_MASK		(0xf<<16)
#define S5P_CLKDIV3_FIMC0_SHIFT		(16)
#define S5P_CLKDIV3_FIMC1_MASK		(0xf<<20)
#define S5P_CLKDIV3_FIMC1_SHIFT		(20)
#define S5P_CLKDIV3_FIMC2_MASK		(0xf<<24)
#define S5P_CLKDIV3_FIMC2_SHIFT		(24)
#define S5P_CLKDIV3_HDMI_MASK		(0xf<<28)
#define S5P_CLKDIV3_HDMI_SHIFT		(28)

/* CLKDIV4 */
#define S5P_CLKDIV4_PWI_MASK		(0x7<<0)
#define S5P_CLKDIV4_PWI_SHIFT		(0)
#define S5P_CLKDIV4_HCLKD2_MASK		(0x7<<4)
#define S5P_CLKDIV4_HCLKD2_SHIFT	(4)
#define S5P_CLKDIV4_I2SD2_MASK		(0xf<<8)
#define S5P_CLKDIV4_I2SD2_SHIFT		(8)
#define S5P_CLKDIV4_AUDIO0_MASK		(0xf<<12)
#define S5P_CLKDIV4_AUDIO0_SHIFT	(12)
#define S5P_CLKDIV4_AUDIO1_MASK		(0xf<<16)
#define S5P_CLKDIV4_AUDIO1_SHIFT	(16)
#define S5P_CLKDIV4_AUDIO2_MASK		(0xf<<20)
#define S5P_CLKDIV4_AUDIO2_SHIFT	(20)


/* HCLKD0/PCLKD0 Clock Gate 0 Registers */
#define S5P_CLKGATE_D00_INTC		(1<<0)
#define S5P_CLKGATE_D00_TZIC		(1<<1)
#define S5P_CLKGATE_D00_CFCON		(1<<2)
#define S5P_CLKGATE_D00_MDMA		(1<<3)
#define S5P_CLKGATE_D00_G2D		(1<<4)
#define S5P_CLKGATE_D00_SECSS		(1<<5)
#define S5P_CLKGATE_D00_CSSYS		(1<<6)

/* HCLKD0/PCLKD0 Clock Gate 1 Registers */
#define S5P_CLKGATE_D01_DMC		(1<<0)
#define S5P_CLKGATE_D01_SROMC		(1<<1)
#define S5P_CLKGATE_D01_ONENAND		(1<<2)
#define S5P_CLKGATE_D01_NFCON		(1<<3)
#define S5P_CLKGATE_D01_INTMEM		(1<<4)
#define S5P_CLKGATE_D01_EBI		(1<<5)

/* PCLKD0 Clock Gate 2 Registers */
#define S5P_CLKGATE_D02_SECKEY		(1<<1)
#define S5P_CLKGATE_D02_SDM		(1<<2)

/* HCLKD1/PCLKD1 Clock Gate 0 Registers */
#define S5P_CLKGATE_D10_PDMA0		(1<<0)
#define S5P_CLKGATE_D10_PDMA1		(1<<1)
#define S5P_CLKGATE_D10_USBHOST		(1<<2)
#define S5P_CLKGATE_D10_USBOTG		(1<<3)
#define S5P_CLKGATE_D10_MODEMIF		(1<<4)
#define S5P_CLKGATE_D10_HSMMC0		(1<<5)
#define S5P_CLKGATE_D10_HSMMC1		(1<<6)
#define S5P_CLKGATE_D10_HSMMC2		(1<<7)

/* HCLKD1/PCLKD1 Clock Gate 1 Registers */
#define S5P_CLKGATE_D11_LCD		(1<<0)
#define S5P_CLKGATE_D11_ROTATOR		(1<<1)
#define S5P_CLKGATE_D11_FIMC0		(1<<2)
#define S5P_CLKGATE_D11_FIMC1		(1<<3)
#define S5P_CLKGATE_D11_FIMC2		(1<<4)
#define S5P_CLKGATE_D11_JPEG		(1<<5)
#define S5P_CLKGATE_D11_DSI		(1<<6)
#define S5P_CLKGATE_D11_CSI		(1<<7)
#define S5P_CLKGATE_D11_G3D		(1<<8)

/* HCLKD1/PCLKD1 Clock Gate 2 Registers */
#define S5P_CLKGATE_D12_TV		(1<<0)
#define S5P_CLKGATE_D12_VP		(1<<1)
#define S5P_CLKGATE_D12_MIXER		(1<<2)
#define S5P_CLKGATE_D12_HDMI		(1<<3)
#define S5P_CLKGATE_D12_MFC		(1<<4)

/* HCLKD1/PCLKD1 Clock Gate 3 Registers */
#define S5P_CLKGATE_D13_CHIPID		(1<<0)
#define S5P_CLKGATE_D13_GPIO		(1<<1)
#define S5P_CLKGATE_D13_APC		(1<<2)
#define S5P_CLKGATE_D13_IEC		(1<<3)
#define S5P_CLKGATE_D13_PWM		(1<<6)
#define S5P_CLKGATE_D13_SYSTIMER	(1<<7)
#define S5P_CLKGATE_D13_WDT		(1<<8)
#define S5P_CLKGATE_D13_RTC		(1<<9)

/* HCLKD1/PCLKD1 Clock Gate 4 Registers */
#define S5P_CLKGATE_D14_UART0		(1<<0)
#define S5P_CLKGATE_D14_UART1		(1<<1)
#define S5P_CLKGATE_D14_UART2		(1<<2)
#define S5P_CLKGATE_D14_UART3		(1<<3)
#define S5P_CLKGATE_D14_IIC		(1<<4)
#define S5P_CLKGATE_D14_HDMI_IIC	(1<<5)
#define S5P_CLKGATE_D14_SPI0		(1<<6)
#define S5P_CLKGATE_D14_SPI1		(1<<7)
#define S5P_CLKGATE_D14_SPI2		(1<<8)
#define S5P_CLKGATE_D14_IRDA		(1<<9)
#define S5P_CLKGATE_D14_CCAN0		(1<<10)
#define S5P_CLKGATE_D14_CCAN1		(1<<11)
#define S5P_CLKGATE_D14_HSITX		(1<<12)
#define S5P_CLKGATE_D14_HSIRX		(1<<13)

/* HCLKD1/PCLKD1 Clock Gate 5 Registers */
#define S5P_CLKGATE_D15_IIS0		(1<<0)
#define S5P_CLKGATE_D15_IIS1		(1<<1)
#define S5P_CLKGATE_D15_IIS2		(1<<2)
#define S5P_CLKGATE_D15_AC97		(1<<3)
#define S5P_CLKGATE_D15_PCM0		(1<<4)
#define S5P_CLKGATE_D15_PCM1		(1<<5)
#define S5P_CLKGATE_D15_SPDIF		(1<<6)
#define S5P_CLKGATE_D15_TSADC		(1<<7)
#define S5P_CLKGATE_D15_KEYIF		(1<<8)
#define S5P_CLKGATE_D15_CG		(1<<9)

/* HCLKD2 Clock Gate 0 Registers */
#define S5P_CLKGATE_D20_HCLKD2		(1<<0)
#define S5P_CLKGATE_D20_I2SD2		(1<<1)

/* Special Clock Gate 0 Registers */
#define	S5P_CLKGATE_SCLK0_HPM		(1<<0)
#define	S5P_CLKGATE_SCLK0_PWI		(1<<1)
#define	S5P_CLKGATE_SCLK0_ONENAND	(1<<2)
#define	S5P_CLKGATE_SCLK0_UART		(1<<3)
#define	S5P_CLKGATE_SCLK0_SPI0		(1<<4)
#define	S5P_CLKGATE_SCLK0_SPI1		(1<<5)
#define	S5P_CLKGATE_SCLK0_SPI2		(1<<6)
#define	S5P_CLKGATE_SCLK0_SPI0_48	(1<<7)
#define	S5P_CLKGATE_SCLK0_SPI1_48	(1<<8)
#define	S5P_CLKGATE_SCLK0_SPI2_48	(1<<9)
#define	S5P_CLKGATE_SCLK0_IRDA		(1<<10)
#define	S5P_CLKGATE_SCLK0_USBHOST	(1<<11)
#define	S5P_CLKGATE_SCLK0_MMC0		(1<<12)
#define	S5P_CLKGATE_SCLK0_MMC1		(1<<13)
#define	S5P_CLKGATE_SCLK0_MMC2		(1<<14)
#define	S5P_CLKGATE_SCLK0_MMC0_48	(1<<15)
#define	S5P_CLKGATE_SCLK0_MMC1_48	(1<<16)
#define	S5P_CLKGATE_SCLK0_MMC2_48	(1<<17)

/* Special Clock Gate 1 Registers */
#define	S5P_CLKGATE_SCLK1_LCD		(1<<0)
#define	S5P_CLKGATE_SCLK1_FIMC0		(1<<1)
#define	S5P_CLKGATE_SCLK1_FIMC1		(1<<2)
#define	S5P_CLKGATE_SCLK1_FIMC2		(1<<3)
#define	S5P_CLKGATE_SCLK1_TV54		(1<<4)
#define	S5P_CLKGATE_SCLK1_VDAC54	(1<<5)
#define	S5P_CLKGATE_SCLK1_MIXER		(1<<6)
#define	S5P_CLKGATE_SCLK1_HDMI		(1<<7)
#define	S5P_CLKGATE_SCLK1_AUDIO0	(1<<8)
#define	S5P_CLKGATE_SCLK1_AUDIO1	(1<<9)
#define	S5P_CLKGATE_SCLK1_AUDIO2	(1<<10)
#define	S5P_CLKGATE_SCLK1_SPDIF		(1<<11)
#define	S5P_CLKGATE_SCLK1_CAM		(1<<12)

/* register for power management */
#define S5P_PWR_CFG 		S5P_CLKREG(0x8000)
#define S5P_EINT_WAKEUP_MASK 	S5P_CLKREG(0x8004)
#define S5P_NORMAL_CFG 		S5P_CLKREG(0x8010)
#define S5P_STOP_CFG 		S5P_CLKREG(0x8014)
#define S5P_SLEEP_CFG 		S5P_CLKREG(0x8018)
#define S5P_STOP_MEM_CFG 	S5P_CLKREG(0x801C)
#define S5P_OSC_FREQ 		S5P_CLKREG(0x8100)
#define S5P_OSC_STABLE 		S5P_CLKREG(0x8104)
#define S5P_PWR_STABLE 		S5P_CLKREG(0x8108)
#define S5P_MTC_STABLE 		S5P_CLKREG(0x8110)
#define S5P_CLAMP_STABLE 	S5P_CLKREG(0x8114)
#define S5P_OTHERS 		S5P_CLKREG(0x8200)
#define S5P_RST_STAT 		S5P_CLKREG(0x8300)
#define S5P_WAKEUP_STAT 	S5P_CLKREG(0x8304)
#define S5P_BLK_PWR_STAT 	S5P_CLKREG(0x8308)
#define S5P_INFORM0 		S5P_CLKREG(0x8400)
#define S5P_INFORM1 		S5P_CLKREG(0x8404)
#define S5P_INFORM2 		S5P_CLKREG(0x8408)
#define S5P_INFORM3 		S5P_CLKREG(0x840C)
#define S5P_INFORM4 		S5P_CLKREG(0x8410)
#define S5P_INFORM5 		S5P_CLKREG(0x8414)
#define S5P_INFORM6 		S5P_CLKREG(0x8418)
#define S5P_INFORM7 		S5P_CLKREG(0x841C)
#define S5P_DCGIDX_MAP0 	S5P_CLKREG(0x8500)
#define S5P_DCGIDX_MAP1 	S5P_CLKREG(0x8504)
#define S5P_DCGIDX_MAP2 	S5P_CLKREG(0x8508)
#define S5P_DCGPERF_MAP0 	S5P_CLKREG(0x850C)
#define S5P_DCGPERF_MAP1 	S5P_CLKREG(0x8510)
#define S5P_DVCIDX_MAP 		S5P_CLKREG(0x8514)
#define S5P_FREQ_CPU 		S5P_CLKREG(0x8518)
#define S5P_FREQ_DPM 		S5P_CLKREG(0x851C)
#define S5P_DVSEMCLK_EN 	S5P_CLKREG(0x8520)
#define S5P_APLL_CON_L8 	S5P_CLKREG(0x8600)
#define S5P_APLL_CON_L7 	S5P_CLKREG(0x8604)
#define S5P_APLL_CON_L6 	S5P_CLKREG(0x8608)
#define S5P_APLL_CON_L5 	S5P_CLKREG(0x860C)
#define S5P_APLL_CON_L4 	S5P_CLKREG(0x8610)
#define S5P_APLL_CON_L3 	S5P_CLKREG(0x8614)
#define S5P_APLL_CON_L2 	S5P_CLKREG(0x8618)
#define S5P_APLL_CON_L1 	S5P_CLKREG(0x861C)
#define S5P_APLL_CON_L(x) 	S5P_CLKREG(0x8600 + (8-x)*4)
#define S5P_IEM_CONTROL 	S5P_CLKREG(0x8620)
#define S5P_CLKDIV_IEM_L8 	S5P_CLKREG(0x8700)
#define S5P_CLKDIV_IEM_L7 	S5P_CLKREG(0x8704)
#define S5P_CLKDIV_IEM_L6 	S5P_CLKREG(0x8708)
#define S5P_CLKDIV_IEM_L5 	S5P_CLKREG(0x870C)
#define S5P_CLKDIV_IEM_L4 	S5P_CLKREG(0x8710)
#define S5P_CLKDIV_IEM_L3 	S5P_CLKREG(0x8714)
#define S5P_CLKDIV_IEM_L2 	S5P_CLKREG(0x8718)
#define S5P_CLKDIV_IEM_L1 	S5P_CLKREG(0x871C)
#define S5P_CLKDIV_IEM_L(x) 	S5P_CLKREG(0x8700 + (8-x)*4)
#define S5P_IEM_HPMCLK_DIV 	S5P_CLKREG(0x8724)

#define S5P_SWRESET		S5P_CLKREG(0x100000)
#define S5P_OND_SWRESET		S5P_CLKREG(0x100008)
#define S5P_GEN_CTRL		S5P_CLKREG(0x100100)
#define S5P_GEN_STATUS		S5P_CLKREG(0x100104)
#define S5P_MEM_SYS_CFG		S5P_CLKREG(0x100200)
#define S5P_CAM_MUX_SEL		S5P_CLKREG(0x100300)
#define S5P_MIXER_OUT_SEL	S5P_CLKREG(0x100304)
#define S5P_LPMP_MODE_SEL	S5P_CLKREG(0x100308)
#define S5P_MIPI_PHY_CON0	S5P_CLKREG(0x100400)
#define S5P_MIPI_PHY_CON1	S5P_CLKREG(0x100414)
#define S5P_HDMI_PHY_CON0	S5P_CLKREG(0x100420)

#define S5P_CFG_WFI_CLEAN	~(3<<5)
#define S5P_CFG_WFI_IDLE	(1<<5)
#define S5P_CFG_WFI_STOP	(2<<5)
#define S5P_CFG_WFI_SLEEP	(3<<5)

#define S5P_OTHER_SYS_INT	24
#define S5P_OTHER_STA_TYPE	23
#define STA_TYPE_EXPON		0
#define STA_TYPE_SFR		1

#define S5P_PWR_STA_EXP_SCALE	0
#define S5P_PWR_STA_CNT		4

#define S5P_PWR_STABLE_COUNT	85500

#define S5P_SLEEP_CFG_OSC_EN	0

/* OTHERS Resgister */
#define S5P_OTHERS_USB_SIG_MASK 	(1 << 16)
#define S5P_OTHERS_MIPI_DPHY_EN		(1 << 28)

/* MIPI D-PHY Control Register 0 */
#define S5P_MIPI_PHY_CON0_M_RESETN	(1 << 1)
#define S5P_MIPI_PHY_CON0_S_RESETN	(1 << 0)

/* IEM registers */
#define	S5P_IECDPCCR			S5P_IECREG(0x000)
#define	S5P_IECDVSEMSTR			S5P_IECREG(0x004)
#define	S5P_IECDPCTGTPERF		S5P_IECREG(0x008)
#define	S5P_IECDPCCRNTPERF		S5P_IECREG(0x00C)
#define	S5P_IECIMSC			S5P_IECREG(0x010)
#define	S5P_IECRIS			S5P_IECREG(0x014)
#define	S5P_IECMIS			S5P_IECREG(0x018)
#define	S5P_IECICR			S5P_IECREG(0x01C)
#define	S5P_IECCFGCPUFREQ		S5P_IECREG(0x020)
#define	S5P_IECDPMFREQ			S5P_IECREG(0x024)
#define	S5P_IECCFGDCGIDXMAP00		S5P_IECREG(0x040)
#define	S5P_IECCFGDCGIDXMAP32		S5P_IECREG(0x044)
#define	S5P_IECCFGDCGIDXMAP64		S5P_IECREG(0x048)
#define	S5P_IECCFGDVCIDXMAP		S5P_IECREG(0x04C)
#define	S5P_IECCFGDCGPERFMAP0		S5P_IECREG(0x060)
#define	S5P_IECCFGDCGPERFMAP4		S5P_IECREG(0x064)
#define	S5P_IECDPMCR			S5P_IECREG(0x100)
#define	S5P_IECDPM2RATE			S5P_IECREG(0x108)
#define	S5P_IECDPM3RATE			S5P_IECREG(0x10C)
#define	S5P_IECDPMILO			S5P_IECREG(0x180)
#define	S5P_IECDPM1H1			S5P_IECREG(0x184)
#define	S5P_IECDPM2LO			S5P_IECREG(0x188)
#define	S5P_IECDPM2HI			S5P_IECREG(0x18C)
#define	S5P_IECDPM3LO			S5P_IECREG(0x190)
#define	S5P_IECDPM3HI			S5P_IECREG(0x194)
#define	S5P_IECPeriphID4		S5P_IECREG(0xFD0)
#define	S5P_IECPeriphID5		S5P_IECREG(0xFD4)
#define	S5P_IECPeriphID6		S5P_IECREG(0xFD8)
#define	S5P_IECPeriphID7		S5P_IECREG(0xFDC)
#define	S5P_IECPeriphID0		S5P_IECREG(0xFE0)
#define	S5P_IECPeriphID1		S5P_IECREG(0xFE4)
#define	S5P_IECPeriphID2		S5P_IECREG(0xFE8)
#define	S5P_IECPeriphID3		S5P_IECREG(0xFEC)
#define	S5P_IECID0			S5P_IECREG(0xFF0)
#define	S5P_IECID1			S5P_IECREG(0xFF4)
#define	S5P_IECID2			S5P_IECREG(0xFF8)
#define	S5P_IECID3			S5P_IECREG(0xFFC)


#define	S5P_APC_PWICMD			S5P_APCREG(0x000)
#define	S5P_APC_PWIDATAWR		S5P_APCREG(0x004)
#define	S5P_APC_PWIDATARD		S5P_APCREG(0x008)
#define	S5P_APC_CONTROL			S5P_APCREG(0x010)
#define	S5P_APC_STATUS			S5P_APCREG(0x014)
#define	S5P_APC_MINVDD_LIMIT		S5P_APCREG(0x018)
#define	S5P_APC_VDDCHK			S5P_APCREG(0x01C)
#define	S5P_APC_VDDCHKD			S5P_APCREG(0x020)
#define	S5P_APC_PREDLYSEL		S5P_APCREG(0x024)
#define	S5P_APC_IMASK			S5P_APCREG(0x028)
#define	S5P_APC_ISTATUS			S5P_APCREG(0x02C)
#define	S5P_APC_ICLEAR			S5P_APCREG(0x030)
#define	S5P_APC_UNSH_NOISE		S5P_APCREG(0x034)
#define	S5P_APC_WKUP_DLY		S5P_APCREG(0x038)
#define	S5P_APC_SLK_SMP			S5P_APCREG(0x03C)
#define	S5P_APC_CLKDIV_PWICLK		S5P_APCREG(0x040)
#define	S5P_APC_OVSHT_LMT		S5P_APCREG(0x050)
#define	S5P_APC_CLP_CTRL		S5P_APCREG(0x054)
#define	S5P_APC_SS_SRATE		S5P_APCREG(0x058)
#define	S5P_APC_IGAIN4			S5P_APCREG(0x05C)
#define	S5P_APC_IGAIN1			S5P_APCREG(0x060)
#define	S5P_APC_IGAIN2			S5P_APCREG(0x064)
#define	S5P_APC_IGAIN3			S5P_APCREG(0x068)
#define	S5P_APC_PL1_CALCODE		S5P_APCREG(0x080)
#define	S5P_APC_PL2_CALCODE		S5P_APCREG(0x084)
#define	S5P_APC_PL3_CALCODE		S5P_APCREG(0x088)
#define	S5P_APC_PL4_CALCODE		S5P_APCREG(0x08C)
#define	S5P_APC_PL5_CALCODE		S5P_APCREG(0x090)
#define	S5P_APC_PL6_CALCODE		S5P_APCREG(0x094)
#define	S5P_APC_PL7_CALCODE		S5P_APCREG(0x098)
#define	S5P_APC_PL8_CALCODE		S5P_APCREG(0x09C)
#define	S5P_APC_PL1_COREVDD		S5P_APCREG(0x0A0)
#define	S5P_APC_PL2_COREVDD		S5P_APCREG(0x0A4)
#define	S5P_APC_PL3_COREVDD		S5P_APCREG(0x0A8)
#define	S5P_APC_PL4_COREVDD		S5P_APCREG(0x0AC)
#define	S5P_APC_PL5_COREVDD		S5P_APCREG(0x0B0)
#define	S5P_APC_PL6_COREVDD		S5P_APCREG(0x0B4)
#define	S5P_APC_PL7_COREVDD		S5P_APCREG(0x0B8)
#define	S5P_APC_PL8_COREVDD		S5P_APCREG(0x0BC)
#define	S5P_APC_RET_VDD			S5P_APCREG(0x0C0)
#define	S5P_APC_ITSTOP3			S5P_APCREG(0x0C4)
#define	S5P_APC_DBG_DLYCODE		S5P_APCREG(0x0E0)
#define	S5P_APC_REV			S5P_APCREG(0x0FC)


#elif defined(CONFIG_CPU_S5PC110)

#define S5P_APLL_LOCK		S5P_CLKREG(0x00)
#define S5P_MPLL_LOCK		S5P_CLKREG(0x08)
#define S5P_EPLL_LOCK		S5P_CLKREG(0x10)
#define S5P_VPLL_LOCK		S5P_CLKREG(0x20)

#define S5P_APLL_CON		S5P_CLKREG(0x100)
#define S5P_MPLL_CON		S5P_CLKREG(0x108)
#define S5P_EPLL_CON		S5P_CLKREG(0x110)
#define S5P_VPLL_CON		S5P_CLKREG(0x120)

#define S5P_CLK_SRC0		S5P_CLKREG(0x200)
#define S5P_CLK_SRC1		S5P_CLKREG(0x204)
#define S5P_CLK_SRC2		S5P_CLKREG(0x208)
#define S5P_CLK_SRC3		S5P_CLKREG(0x20C)
#define S5P_CLK_SRC4		S5P_CLKREG(0x210)
#define S5P_CLK_SRC5		S5P_CLKREG(0x214)
#define S5P_CLK_SRC6		S5P_CLKREG(0x218)

#define S5P_CLK_SRC_MASK0	S5P_CLKREG(0x280)
#define S5P_CLK_SRC_MASK1	S5P_CLKREG(0x284)

#define S5P_CLK_DIV0		S5P_CLKREG(0x300)
#define S5P_CLK_DIV1		S5P_CLKREG(0x304)
#define S5P_CLK_DIV2		S5P_CLKREG(0x308)
#define S5P_CLK_DIV3		S5P_CLKREG(0x30C)
#define S5P_CLK_DIV4		S5P_CLKREG(0x310)
#define S5P_CLK_DIV5		S5P_CLKREG(0x314)
#define S5P_CLK_DIV6		S5P_CLKREG(0x318)
#define S5P_CLK_DIV7		S5P_CLKREG(0x31C)

#define S5P_CLKGATE_MAIN0	S5P_CLKREG(0x400)
#define S5P_CLKGATE_MAIN1	S5P_CLKREG(0x404)
#define S5P_CLKGATE_MAIN2	S5P_CLKREG(0x408)

#define S5P_CLKGATE_PERI0	S5P_CLKREG(0x420)
#define S5P_CLKGATE_PERI1	S5P_CLKREG(0x424)

#define S5P_CLKGATE_SCLK0	S5P_CLKREG(0x440)
#define S5P_CLKGATE_SCLK1	S5P_CLKREG(0x444)

#define S5P_SCLKGATE0		S5P_CLKGATE_SCLK0
#define S5P_SCLKGATE1		S5P_CLKGATE_SCLK1

#define S5P_CLKGATE_IP0		S5P_CLKREG(0x460)
#define S5P_CLKGATE_IP1		S5P_CLKREG(0x464)
#define S5P_CLKGATE_IP2		S5P_CLKREG(0x468)
#define S5P_CLKGATE_IP3		S5P_CLKREG(0x46C)
#define S5P_CLKGATE_IP4		S5P_CLKREG(0x470)

#define S5P_CLKGATE_BLOCK	S5P_CLKREG(0x480)
#define S5P_CLKGATE_BUS0	S5P_CLKREG(0x484)
#define S5P_CLKGATE_BUS1	S5P_CLKREG(0x488)

#define S5P_CLK_OUT		S5P_CLKREG(0x500)
#define S5P_MDNIE_SEL		S5P_CLKREG(0x7008)

#define S5P_EPLL_EN     (1<<31)
#define S5P_EPLL_MASK   0xffffffff 
#define S5P_EPLLVAL(_m,_p,_s)   ((_m) << 16 | ((_p) << 8) | ((_s)))

/* CLKSRC0 */
#define S5P_CLKSRC0_APLL_MASK		(0x1<<0)
#define S5P_CLKSRC0_APLL_SHIFT		(0)
#define S5P_CLKSRC0_MPLL_MASK		(0x1<<4)
#define S5P_CLKSRC0_MPLL_SHIFT		(4)
#define S5P_CLKSRC0_EPLL_MASK		(0x1<<8)
#define S5P_CLKSRC0_EPLL_SHIFT		(8)
#define S5P_CLKSRC0_VPLL_MASK		(0x1<<12)
#define S5P_CLKSRC0_VPLL_SHIFT		(12)
#define S5P_CLKSRC0_MUX200_MASK		(0x1<<16)
#define S5P_CLKSRC0_MUX200_SHIFT	(16)
#define S5P_CLKSRC0_MUX166_MASK		(0x1<<20)
#define S5P_CLKSRC0_MUX166_SHIFT	(20)
#define S5P_CLKSRC0_MUX133_MASK		(0x1<<24)
#define S5P_CLKSRC0_MUX133_SHIFT	(24)
#define S5P_CLKSRC0_ONENAND_MASK	(0x1<<28)
#define S5P_CLKSRC0_ONENAND_SHIFT	(28)

/* CLKSRC1 */
#define S5P_CLKSRC1_HDMI_MASK		(0x1<<0)
#define S5P_CLKSRC1_HDMI_SHIFT		(0)
#define S5P_CLKSRC1_MIXER_MASK		(0x1<<4)
#define S5P_CLKSRC1_MIXER_SHIFT		(4)
#define S5P_CLKSRC1_DAC_MASK		(0x1<<8)
#define S5P_CLKSRC1_DAC_SHIFT		(8)
#define S5P_CLKSRC1_CAM0_MASK		(0xF<<12)
#define S5P_CLKSRC1_CAM0_SHIFT		(12)
#define S5P_CLKSRC1_CAM1_MASK		(0xF<<16)
#define S5P_CLKSRC1_CAM1_SHIFT		(16)
#define S5P_CLKSRC1_FIMD_MASK		(0xF<<20)
#define S5P_CLKSRC1_FIMD_SHIFT		(20)
#define S5P_CLKSRC1_CSIS_MASK		(0xF<<24)
#define S5P_CLKSRC1_CSIS_SHIFT		(24)
#define S5P_CLKSRC1_VPLLSRC_MASK	(0x1<<28)
#define S5P_CLKSRC1_VPLLSRC_SHIFT	(28)

/* CLKSRC2 */
#define S5P_CLKSRC2_G3D_MASK		(0x3<<0)
#define S5P_CLKSRC2_G3D_SHIFT		(0)
#define S5P_CLKSRC2_MFC_MASK		(0x3<<4)
#define S5P_CLKSRC2_MFC_SHIFT		(4)

/* CLKSRC3 */
#define S5P_CLKSRC3_MDNIE_MASK		(0xF<<0)
#define S5P_CLKSRC3_MDNIE_SHIFT		(0)
#define S5P_CLKSRC3_MDINE_PWMCLK_MASK	(0xF<<4)
#define S5P_CLKSRC3_MDINE_PWMCLK_SHIFT	(4)
#define S5P_CLKSRC3_FIMC0_LCLK_MASK	(0xF<<12)
#define S5P_CLKSRC3_FIMC0_LCLK_SHIFT	(12)
#define S5P_CLKSRC3_FIMC1_LCLK_MASK	(0xF<<16)
#define S5P_CLKSRC3_FIMC1_LCLK_SHIFT	(16)
#define S5P_CLKSRC3_FIMC2_LCLK_MASK	(0xF<<20)
#define S5P_CLKSRC3_FIMC2_LCLK_SHIFT	(20)

/* CLKSRC4 */
#define S5P_CLKSRC4_MMC0_MASK		(0xF<<0)
#define S5P_CLKSRC4_MMC0_SHIFT		(0)
#define S5P_CLKSRC4_MMC1_MASK		(0xF<<4)
#define S5P_CLKSRC4_MMC1_SHIFT		(4)
#define S5P_CLKSRC4_MMC2_MASK		(0xF<<8)
#define S5P_CLKSRC4_MMC2_SHIFT		(8)
#define S5P_CLKSRC4_MMC3_MASK		(0xF<<12)
#define S5P_CLKSRC4_MMC3_SHIFT		(12)
#define S5P_CLKSRC4_UART0_MASK		(0xF<<16)
#define S5P_CLKSRC4_UART0_SHIFT		(16)
#define S5P_CLKSRC4_UART1_MASK		(0xF<<20)
#define S5P_CLKSRC4_UART1_SHIFT		(20)
#define S5P_CLKSRC4_UART2_MASK		(0xF<<24)
#define S5P_CLKSRC4_UART2_SHIFT		(24)
#define S5P_CLKSRC4_UART3_MASK		(0xF<<28)
#define S5P_CLKSRC4_UART3_SHIFT		(28)

/* CLKSRC5 */
#define S5P_CLKSRC5_SPI0_MASK		(0xF<<0)
#define S5P_CLKSRC5_SPI0_SHIFT		(0)
#define S5P_CLKSRC5_SPI1_MASK		(0xF<<4)
#define S5P_CLKSRC5_SPI1_SHIFT		(4)
#define S5P_CLKSRC5_SPI2_MASK		(0xF<<8)
#define S5P_CLKSRC5_SPI2_SHIFT		(8)
#define S5P_CLKSRC5_PWM_MASK		(0xF<<12)
#define S5P_CLKSRC5_PWM_SHIFT		(12)

/* CLKSRC6 */
#define S5P_CLKSRC6_AUDIO0_MASK		(0xF<<0)
#define S5P_CLKSRC6_AUDIO0_SHIFT	(0)
#define S5P_CLKSRC6_AUDIO1_MASK		(0xF<<4)
#define S5P_CLKSRC6_AUDIO1_SHIFT	(4)
#define S5P_CLKSRC6_AUDIO2_MASK		(0xF<<8)
#define S5P_CLKSRC6_AUDIO02_SHIFT	(8)
#define S5P_CLKSRC6_SPDIF_MASK		(0x3<<12)
#define S5P_CLKSRC6_SPDIF_SHIFT		(12)
#define S5P_CLKSRC6_HPM_MASK		(0x1<<16)
#define S5P_CLKSRC6_HPM_SHIFT		(16)
#define S5P_CLKSRC6_PWI_MASK		(0xF<<20)
#define S5P_CLKSRC6_PWI_SHIFT		(20)
#define S5P_CLKSRC6_ONEDRAM_MASK	(0x3<<24)
#define S5P_CLKSRC6_ONEDRAM_SHIFT	(24)

/* CLKSRC_MASK0 - To be defined */
#define S5P_CLKSRC_MASK0_HDMI		(1<<0)
#define S5P_CLKSRC_MASK0_MIXER		(1<<1)

/* CLKSRC_MASK1 - To be defined */


/* CLKDIV0 */
#define S5P_CLKDIV0_APLL_MASK		(0x7<<0)
#define S5P_CLKDIV0_APLL_SHIFT		(0)
#define S5P_CLKDIV0_A2M_MASK		(0x7<<4)
#define S5P_CLKDIV0_A2M_SHIFT		(4)
#define S5P_CLKDIV0_HCLK200_MASK	(0x7<<8)
#define S5P_CLKDIV0_HCLK200_SHIFT	(8)
#define S5P_CLKDIV0_PCLK100_MASK	(0x7<<12)
#define S5P_CLKDIV0_PCLK100_SHIFT	(12)
#define S5P_CLKDIV0_HCLK166_MASK	(0xF<<16)
#define S5P_CLKDIV0_HCLK166_SHIFT	(16)
#define S5P_CLKDIV0_PCLK83_MASK		(0x7<<20)
#define S5P_CLKDIV0_PCLK83_SHIFT	(20)
#define S5P_CLKDIV0_HCLK133_MASK	(0xF<<24)
#define S5P_CLKDIV0_HCLK133_SHIFT	(24)
#define S5P_CLKDIV0_PCLK66_MASK		(0x7<<28)
#define S5P_CLKDIV0_PCLK66_SHIFT	(28)

/* CLKDIV1 */
#define S5P_CLKDIV1_TBLK_MASK		(0xF<<0)
#define S5P_CLKDIV1_TBLK_SHIFT		(0)
#define S5P_CLKDIV1_FIMC_MASK		(0xF<<8)
#define S5P_CLKDIV1_FIMC_SHIFT		(8)
#define S5P_CLKDIV1_CAM0_MASK		(0xF<<12)
#define S5P_CLKDIV1_CAM0_SHIFT		(12)
#define S5P_CLKDIV1_CAM1_MASK		(0xF<<16)
#define S5P_CLKDIV1_CAM1_SHIFT		(16)
#define S5P_CLKDIV1_FIMD_MASK		(0xF<<20)
#define S5P_CLKDIV1_FIMD_SHIFT		(20)
#define S5P_CLKDIV1_CSIS_MASK		(0xF<<28)
#define S5P_CLKDIV1_CSIS_SHIFT		(28)

/* CLKDIV2 */
#define S5P_CLKDIV2_G3D_MASK		(0xF<<0)
#define S5P_CLKDIV2_G3D_SHIFT		(0)
#define S5P_CLKDIV2_MFC_MASK		(0xF<<4)
#define S5P_CLKDIV2_MFC_SHIFT		(4)

/* CLKDIV3 */
#define S5P_CLKDIV3_MDINE_MASK		(0xf<<0)
#define S5P_CLKDIV3_MDINE_SHIFT		(0)
#define S5P_CLKDIV3_MDINE_PWM_MASK	(0x3f<<4)
#define S5P_CLKDIV3_MDINE_PWM_SHIFT	(4)
#define S5P_CLKDIV3_FIMC0_LCLK_MASK	(0xf<<12)
#define S5P_CLKDIV3_FIMC0_LCLK_SHIFT	(12)
#define S5P_CLKDIV3_FIMC1_LCLK_MASK	(0xf<<16)
#define S5P_CLKDIV3_FIMC1_LCLK_SHIFT	(16)
#define S5P_CLKDIV3_FIMC2_LCLK_MASK	(0xf<<20)
#define S5P_CLKDIV3_FIMC2_LCLK_SHIFT	(20)

/* CLKDIV4 */
#define S5P_CLKDIV4_MMC0_MASK		(0xF<<0)
#define S5P_CLKDIV4_MMC0_SHIFT		(0)
#define S5P_CLKDIV4_MMC1_MASK		(0xF<<4)
#define S5P_CLKDIV4_MMC1_SHIFT		(4)
#define S5P_CLKDIV4_MMC2_MASK		(0xF<<8)
#define S5P_CLKDIV4_MMC2_SHIFT		(8)
#define S5P_CLKDIV4_MMC3_MASK		(0xF<<12)
#define S5P_CLKDIV4_MMC3_SHIFT		(12)
#define S5P_CLKDIV4_UART0_MASK		(0xF<<16)
#define S5P_CLKDIV4_UART0_SHIFT		(16)
#define S5P_CLKDIV4_UART1_MASK		(0xf<<20)
#define S5P_CLKDIV4_UART1_SHIFT		(20)
#define S5P_CLKDIV4_UART2_MASK		(0xf<<24)
#define S5P_CLKDIV4_UART2_SHIFT		(24)
#define S5P_CLKDIV4_UART3_MASK		(0xf<<28)
#define S5P_CLKDIV4_UART3_SHIFT		(28)

/* CLKDIV5 */
#define S5P_CLKDIV5_SPI0_MASK		(0xF<<0)
#define S5P_CLKDIV5_SPI0_SHIFT		(0)
#define S5P_CLKDIV5_SPI1_MASK		(0xF<<4)
#define S5P_CLKDIV5_SPI1_SHIFT		(4)
#define S5P_CLKDIV5_SPI2_MASK		(0xF<<8)
#define S5P_CLKDIV5_SPI2_SHIFT		(8)
#define S5P_CLKDIV5_PWM_MASK		(0xF<<12)
#define S5P_CLKDIV5_PWM_SHIFT		(12)

/* CLKDIV6 */
#define S5P_CLKDIV6_AUDIO0_MASK		(0xF<<0)
#define S5P_CLKDIV6_AUDIO0_SHIFT	(0)
#define S5P_CLKDIV6_AUDIO1_MASK		(0xF<<4)
#define S5P_CLKDIV6_AUDIO1_SHIFT	(4)
#define S5P_CLKDIV6_AUDIO2_MASK		(0xF<<8)
#define S5P_CLKDIV6_AUDIO2_SHIFT	(8)
#define S5P_CLKDIV6_ONENAND_MASK	(0x7<<12)
#define S5P_CLKDIV6_ONENAND_SHIFT	(12)
#define S5P_CLKDIV6_COPY_MASK		(0x7<<16)
#define S5P_CLKDIV6_COPY_SHIFT		(16)
#define S5P_CLKDIV6_HPM_MASK		(0x7<<20)
#define S5P_CLKDIV6_HPM_SHIFT		(20)
#define S5P_CLKDIV6_PWI_MASK		(0xf<<24)
#define S5P_CLKDIV6_PWI_SHIFT		(24)
#define S5P_CLKDIV6_ONEDRAM_MASK	(0xf<<28)
#define S5P_CLKDIV6_ONEDRAM_SHIFT	(28)

/* Clock Gate Main0 Registers */
#define	S5P_CLKGATE_MAIN0_IPC     	(1<<30)
#define S5P_CLKGATE_MAIN0_ROTATOR 	(1<<29)
#define S5P_CLKGATE_MAIN0_JPEG    	(1<<28)
#define S5P_CLKGATE_MAIN0_FIMC2   	(1<<26)
#define S5P_CLKGATE_MAIN0_FIMC1   	(1<<25)
#define S5P_CLKGATE_MAIN0_FIMC0   	(1<<24)
#define S5P_CLKGATE_MAIN0_G3D     	(1<<8)
#define S5P_CLKGATE_MAIN0_IMEM    	(1<<5)
#define S5P_CLKGATE_MAIN0_PDMA1   	(1<<4)
#define S5P_CLKGATE_MAIN0_PDMA0   	(1<<3)
#define S5P_CLKGATE_MAIN0_MDMA    	(1<<2)
#define S5P_CLKGATE_MAIN0_DMC1    	(1<<1)
#define S5P_CLKGATE_MAIN0_DMC0    	(1<<0)


/* Clock Gate Main1 Registers */
#define S5P_CLKGATE_MAIN1_NFCON  	(1<<28)
#define S5P_CLKGATE_MAIN1_SROMC  	(1<<26)
#define S5P_CLKGATE_MAIN1_CFCON  	(1<<25)
#define S5P_CLKGATE_MAIN1_NANDXL 	(1<<24)
#define S5P_CLKGATE_MAIN1_USBHOST	(1<<17)
#define S5P_CLKGATE_MAIN1_USBOTG 	(1<<16)
#define S5P_CLKGATE_MAIN1_TVENC  	(1<<10)
#define S5P_CLKGATE_MAIN1_MIXER  	(1<<9)
#define S5P_CLKGATE_MAIN1_VP     	(1<<8)
#define S5P_CLKGATE_MAIN1_MIE    	(1<<1)
#define S5P_CLKGATE_MAIN1_FIMD   	(1<<0)

/* Clock Gate Main2 Registers */
#define S5P_CLKGATE_MAIN2_TZIC3    	(1<<31)
#define S5P_CLKGATE_MAIN2_TZIC2    	(1<<30)
#define S5P_CLKGATE_MAIN2_TZIC1    	(1<<29)
#define S5P_CLKGATE_MAIN2_TZIC0    	(1<<28)
#define S5P_CLKGATE_MAIN2_VIC3     	(1<<27)
#define S5P_CLKGATE_MAIN2_VIC2    	(1<<26)
#define S5P_CLKGATE_MAIN2_VIC1    	(1<<25)
#define S5P_CLKGATE_MAIN2_VIC0    	(1<<24)
#define S5P_CLKGATE_MAIN2_TSI     	(1<<20)
#define S5P_CLKGATE_MAIN2_HSMMC3  	(1<<19)
#define S5P_CLKGATE_MAIN2_HSMMC2  	(1<<18)
#define S5P_CLKGATE_MAIN2_HSMMC1  	(1<<17)
#define S5P_CLKGATE_MAIN2_HSMMC0  	(1<<16)
#define S5P_CLKGATE_MAIN2_HOSTIF   	(1<<10)
#define S5P_CLKGATE_MAIN2_MODEM    	(1<<9)
#define S5P_CLKGATE_MAIN2_CORESIGHT	(1<<8)
#define S5P_CLKGATE_MAIN2_SDM      	(1<<1)
#define S5P_CLKGATE_MAIN2_SECSS    	(1<<0)


/* Clock Gate Peri0 Registers */
#define S5P_CLKGATE_PERI0_PCM2		(1<<30)
#define S5P_CLKGATE_PERI0_PCM1		(1<<29)
#define S5P_CLKGATE_PERI0_PCM0		(1<<28)
#define S5P_CLKGATE_PERI0_SYSCON	(1<<27)
#define S5P_CLKGATE_PERI0_GPIO		(1<<26)
#define S5P_CLKGATE_PERI0_TSADC		(1<<24)
#define S5P_CLKGATE_PERI0_PWM		(1<<23)
#define S5P_CLKGATE_PERI0_WDT		(1<<22)
#define S5P_CLKGATE_PERI0_KEYIF		(1<<21)
#define S5P_CLKGATE_PERI0_UART3		(1<<20)
#define S5P_CLKGATE_PERI0_UART2		(1<<19)
#define S5P_CLKGATE_PERI0_UART1		(1<<18)
#define S5P_CLKGATE_PERI0_UART0		(1<<17)
#define S5P_CLKGATE_PERI0_SYSTIMER	(1<<16)
#define S5P_CLKGATE_PERI0_RTC		(1<<15)
#define S5P_CLKGATE_PERI0_SPI2		(1<<14)
#define S5P_CLKGATE_PERI0_SPI1		(1<<13)
#define S5P_CLKGATE_PERI0_SPI0		(1<<12)
#define S5P_CLKGATE_PERI0_I2C_HDMI_PHY	(1<<11)
#define S5P_CLKGATE_PERI0_I2C_HDMI_DDC	(1<<10)
#define S5P_CLKGATE_PERI0_I2C2		(1<<9)
#define S5P_CLKGATE_PERI0_I2C1		(1<<8)
#define S5P_CLKGATE_PERI0_I2C0		(1<<7)
#define S5P_CLKGATE_PERI0_I2S2		(1<<6)
#define S5P_CLKGATE_PERI0_I2S1		(1<<5)
#define S5P_CLKGATE_PERI0_I2S0		(1<<4)	
#define S5P_CLKGATE_PERI0_AC97		(1<<1)
#define S5P_CLKGATE_PERI0_SPDIF		(1<<0)

/* Clock Gate Peri1 Registers */
#define S5P_CLKGATE_PERI1_MFC      	(1<<19)
#define S5P_CLKGATE_PERI1_DMC1     	(1<<18)
#define S5P_CLKGATE_PERI1_DMC0     	(1<<17)
#define S5P_CLKGATE_PERI1_MDMA     	(1<<16)
#define S5P_CLKGATE_PERI1_PDMA1    	(1<<15)
#define S5P_CLKGATE_PERI1_PDMA0    	(1<<14)
#define S5P_CLKGATE_PERI1_CSIS     	(1<<13)
#define S5P_CLKGATE_PERI1_DSIM     	(1<<12)
#define S5P_CLKGATE_PERI1_HDMI     	(1<<11)
#define S5P_CLKGATE_PERI1_CORESIGHT	(1<<10)
#define S5P_CLKGATE_PERI1_SECJTAG  	(1<<9)
#define S5P_CLKGATE_PERI1_TZPC3    	(1<<8)
#define S5P_CLKGATE_PERI1_TZPC2    	(1<<7)
#define S5P_CLKGATE_PERI1_TZPC1    	(1<<6)
#define S5P_CLKGATE_PERI1_TZPC0    	(1<<5)
#define S5P_CLKGATE_PERI1_SDM      	(1<<4)
#define S5P_CLKGATE_PERI1_SECKEY   	(1<<3)
#define S5P_CLKGATE_PERI1_IEM_APC  	(1<<2)
#define S5P_CLKGATE_PERI1_IEM_IEC  	(1<<1)
#define S5P_CLKGATE_PERI1_CHIP_ID 	(1<<0)

/* Special Clock Gate 0 Registers */
#define	S5P_CLKGATE_SCLK0_EBI		(1<<31)
#define	S5P_CLKGATE_SCLK0_NANDXL	(1<<30)
#define	S5P_CLKGATE_SCLK0_MFC		(1<<29)
#define	S5P_CLKGATE_SCLK0_G3D		(1<<28)
#define	S5P_CLKGATE_SCLK0_FIMC		(1<<27)
#define	S5P_CLKGATE_SCLK0_CSIS		(1<<26)
#define	S5P_CLKGATE_SCLK0_FIMD		(1<<25)
#define	S5P_CLKGATE_SCLK0_CAM1		(1<<24)
#define	S5P_CLKGATE_SCLK0_CAM0		(1<<23)
#define	S5P_CLKGATE_SCLK0_TVENC		(1<<22)
#define	S5P_CLKGATE_SCLK0_DAC		(1<<21)
#define	S5P_CLKGATE_SCLK0_MIXER		(1<<20)
#define	S5P_CLKGATE_SCLK0_HDMI		(1<<19)
#define	S5P_CLKGATE_SCLK0_SPDIF1	(1<<18)
#define	S5P_CLKGATE_SCLK0_AUDIO2	(1<<17)
#define	S5P_CLKGATE_SCLK0_AUDIO1	(1<<16)
#define	S5P_CLKGATE_SCLK0_AUDIO0	(1<<15)
#define	S5P_CLKGATE_SCLK0_PWM		(1<<14)
#define	S5P_CLKGATE_SCLK0_SPI2		(1<<13)
#define	S5P_CLKGATE_SCLK0_SPI1		(1<<12)
#define	S5P_CLKGATE_SCLK0_SPI0		(1<<11)
#define	S5P_CLKGATE_SCLK0_UART3		(1<<10)
#define	S5P_CLKGATE_SCLK0_UART2		(1<<9)
#define	S5P_CLKGATE_SCLK0_UART1		(1<<8)
#define	S5P_CLKGATE_SCLK0_UART0		(1<<7)
#define	S5P_CLKGATE_SCLK0_MMC3		(1<<6)	
#define	S5P_CLKGATE_SCLK0_MMC2		(1<<5)
#define	S5P_CLKGATE_SCLK0_MMC1		(1<<4)
#define	S5P_CLKGATE_SCLK0_MMC0		(1<<3)
#define	S5P_CLKGATE_SCLK0_PWI		(1<<1)
#define	S5P_CLKGATE_SCLK0_HPM		(1<<0)	

/* Special Clock Gate 1 Registers */
#define	S5P_CLKGATE_SCLK1_FIMC2_LCLK	(1<<5)
#define	S5P_CLKGATE_SCLK1_FIMC1_LCLK	(1<<4)
#define	S5P_CLKGATE_SCLK1_FIMC0_LCLK	(1<<3)
#define	S5P_CLKGATE_SCLK1_MDNIE_PWM	(1<<2)
#define	S5P_CLKGATE_SCLK1_MDNIE		(1<<1)
#define	S5P_CLKGATE_SCLK1_PIXEL		(1<<0)

/* IP Clock Gate 0 Registers */
#define S5P_CLKGATE_IP0_CSIS		(1<<31)	
#define S5P_CLKGATE_IP0_IPC		(1<<30)
#define S5P_CLKGATE_IP0_ROTATOR		(1<<29)
#define S5P_CLKGATE_IP0_JPEG		(1<<28)
#define S5P_CLKGATE_IP0_FIMC2		(1<<26)
#define S5P_CLKGATE_IP0_FIMC1		(1<<25)
#define S5P_CLKGATE_IP0_FIMC0		(1<<24)
#define S5P_CLKGATE_IP0_MFC		(1<<16)
#define S5P_CLKGATE_IP0_G3D		(1<<8)
#define S5P_CLKGATE_IP0_IMEM		(1<<5)
#define S5P_CLKGATE_IP0_PDMA1		(1<<4)
#define S5P_CLKGATE_IP0_PDMA0		(1<<3)
#define S5P_CLKGATE_IP0_MDMA		(1<<2)
#define S5P_CLKGATE_IP0_DMC1		(1<<1)
#define S5P_CLKGATE_IP0_DMC0		(1<<0)

/* IP Clock Gate 1 Registers */
#define S5P_CLKGATE_IP1_NFCON		(1<<28)	
#define S5P_CLKGATE_IP1_SROMC		(1<<26)
#define S5P_CLKGATE_IP1_CFCON		(1<<25)
#define S5P_CLKGATE_IP1_NANDXL		(1<<24)
#define S5P_CLKGATE_IP1_USBHOST		(1<<17)
#define S5P_CLKGATE_IP1_USBOTG		(1<<16)
#define S5P_CLKGATE_IP1_HDMI		(1<<11)
#define S5P_CLKGATE_IP1_TVENC		(1<<10)
#define S5P_CLKGATE_IP1_MIXER		(1<<9)
#define S5P_CLKGATE_IP1_VP		(1<<8)
#define S5P_CLKGATE_IP1_DSIM		(1<<2)
#define S5P_CLKGATE_IP1_MIE		(1<<1)
#define S5P_CLKGATE_IP1_FIMD		(1<<0)

/* IP Clock Gate 2 Registers */
#define S5P_CLKGATE_IP2_TZIC3		(1<<31)	
#define S5P_CLKGATE_IP2_TZIC2		(1<<30)
#define S5P_CLKGATE_IP2_TZIC1		(1<<29)
#define S5P_CLKGATE_IP2_TZIC0		(1<<28)
#define S5P_CLKGATE_IP2_VIC3		(1<<27)
#define S5P_CLKGATE_IP2_VIC2		(1<<26)
#define S5P_CLKGATE_IP2_VIC1		(1<<25)
#define S5P_CLKGATE_IP2_VIC0		(1<<24)
#define S5P_CLKGATE_IP2_TSI		(1<<20)
#define S5P_CLKGATE_IP2_HSMMC3		(1<<19)
#define S5P_CLKGATE_IP2_HSMMC2		(1<<18)
#define S5P_CLKGATE_IP2_HSMMC1		(1<<17)
#define S5P_CLKGATE_IP2_HSMMC0		(1<<16)
#define S5P_CLKGATE_IP2_SECJTAG		(1<<11)
#define S5P_CLKGATE_IP2_HOSTIF		(1<<10)
#define S5P_CLKGATE_IP2_MODEM		(1<<9)
#define S5P_CLKGATE_IP2_CORESIGHT	(1<<8)
#define S5P_CLKGATE_IP2_SDM		(1<<1)
#define S5P_CLKGATE_IP2_SECSS		(1<<0)

/* IP Clock Gate 3 Registers */
#define S5P_CLKGATE_IP3_PCM2		(1<<30)
#define S5P_CLKGATE_IP3_PCM1		(1<<29)
#define S5P_CLKGATE_IP3_PCM0		(1<<28)
#define S5P_CLKGATE_IP3_SYSCON		(1<<27)
#define S5P_CLKGATE_IP3_GPIO		(1<<26)
#define S5P_CLKGATE_IP3_TSADC		(1<<24)
#define S5P_CLKGATE_IP3_PWM		(1<<23)
#define S5P_CLKGATE_IP3_WDT		(1<<22)
#define S5P_CLKGATE_IP3_KEYIF		(1<<21)
#define S5P_CLKGATE_IP3_UART3		(1<<20)
#define S5P_CLKGATE_IP3_UART2		(1<<19)
#define S5P_CLKGATE_IP3_UART1		(1<<18)
#define S5P_CLKGATE_IP3_UART0		(1<<17)
#define S5P_CLKGATE_IP3_SYSTIMER	(1<<16)
#define S5P_CLKGATE_IP3_RTC		(1<<15)
#define S5P_CLKGATE_IP3_SPI2		(1<<14)
#define S5P_CLKGATE_IP3_SPI1		(1<<13)
#define S5P_CLKGATE_IP3_SPI0		(1<<12)
#define S5P_CLKGATE_IP3_I2C_HDMI_PHY	(1<<11)
#define S5P_CLKGATE_IP3_I2C_HDMI_DDC	(1<<10)
#define S5P_CLKGATE_IP3_I2C2		(1<<9)
#define S5P_CLKGATE_IP3_I2C1		(1<<8)
#define S5P_CLKGATE_IP3_I2C0		(1<<7)
#define S5P_CLKGATE_IP3_I2S2		(1<<6)
#define S5P_CLKGATE_IP3_I2S1		(1<<5)
#define S5P_CLKGATE_IP3_I2S0		(1<<4)	
#define S5P_CLKGATE_IP3_AC97		(1<<1)
#define S5P_CLKGATE_IP3_SPDIF		(1<<0)

/* IP Clock Gate 4 Registers */
#define S5P_CLKGATE_IP4_TZPC3		(1<<8)
#define S5P_CLKGATE_IP4_TZPC2		(1<<7)
#define S5P_CLKGATE_IP4_TZPC1		(1<<6)
#define S5P_CLKGATE_IP4_TZPC0		(1<<5)
#define S5P_CLKGATE_IP4_SECKEY		(1<<3)
#define S5P_CLKGATE_IP4_IEM_APC		(1<<2)
#define S5P_CLKGATE_IP4_IEM_IEC		(1<<1)
#define S5P_CLKGATE_IP4_CHIP_ID		(1<<0)

/* Block Clock Gate Registers */
#define S5P_CLKGATE_BLOCK_INTC		(1<<10)
#define S5P_CLKGATE_BLOCK_HSMMC		(1<<9)
#define S5P_CLKGATE_BLOCK_DEBUG		(1<<8)
#define S5P_CLKGATE_BLOCK_SECURITY	(1<<7)
#define S5P_CLKGATE_BLOCK_MEMORY	(1<<6)
#define S5P_CLKGATE_BLOCK_USB		(1<<5)
#define S5P_CLKGATE_BLOCK_TV		(1<<4)
#define S5P_CLKGATE_BLOCK_LCD		(1<<3)
#define S5P_CLKGATE_BLOCK_IMG		(1<<2)
#define S5P_CLKGATE_BLOCK_MFC		(1<<1)
#define S5P_CLKGATE_BLOCK_G3D		(1<<0)

/* Bus Clock Gate Registers (hidden) */


/* register for power management */
#define S5P_PWR_CFG 		S5P_CLKREG(0xC000)
#define S5P_EINT_WAKEUP_MASK 	S5P_CLKREG(0xC004)
#define S5P_WAKEUP_MASK 	S5P_CLKREG(0xC008)
#define S5P_PWR_MODE	 	S5P_CLKREG(0xC00C)
#define S5P_NORMAL_CFG 		S5P_CLKREG(0xC010)
#define S5P_IDLE_CFG 		S5P_CLKREG(0xC020)
#define S5P_STOP_CFG 		S5P_CLKREG(0xC030)
#define S5P_STOP_MEM_CFG 	S5P_CLKREG(0xC034)
#define S5P_SLEEP_CFG 		S5P_CLKREG(0xC040)

#define S5P_OSC_FREQ 		S5P_CLKREG(0xC100)
#define S5P_OSC_STABLE 		S5P_CLKREG(0xC104)
#define S5P_PWR_STABLE 		S5P_CLKREG(0xC108)
#define S5P_MTC_STABLE 		S5P_CLKREG(0xC110)
#define S5P_CLAMP_STABLE 	S5P_CLKREG(0xC114)

#define S5P_WAKEUP_STAT 	S5P_CLKREG(0xC200)
#define S5P_BLK_PWR_STAT 	S5P_CLKREG(0xC204)

#define S5P_OTHERS 		S5P_CLKREG(0xE000)
#define S5P_OM_STAT		S5P_CLKREG(0xE100)

#define S5P_INFORM0 		S5P_CLKREG(0xF000)
#define S5P_INFORM1 		S5P_CLKREG(0xF004)
#define S5P_INFORM2 		S5P_CLKREG(0xF008)
#define S5P_INFORM3 		S5P_CLKREG(0xF00C)
#define S5P_INFORM4 		S5P_CLKREG(0xF010)
#define S5P_INFORM5 		S5P_CLKREG(0xF014)
#define S5P_INFORM6 		S5P_CLKREG(0xF018)
#define S5P_INFORM7 		S5P_CLKREG(0xF01C)

#define S5P_RST_STAT		S5P_CLKREG(0xA000)
#define S5P_OSC_CON		S5P_CLKREG(0x8000)

#define S5P_CFG_WFI_CLEAN	~(3<<8)
#define S5P_CFG_WFI_IDLE	(1<<8)
#define S5P_CFG_WFI_STOP	(2<<8)
#define S5P_CFG_WFI_SLEEP	(3<<8)

#define S5P_OTHER_SYS_INT	24
#define S5P_OTHER_STA_TYPE	23
#define STA_TYPE_EXPON		0
#define STA_TYPE_SFR		1

#define S5P_PWR_STA_EXP_SCALE	0
#define S5P_PWR_STA_CNT		4

#define S5P_PWR_STABLE_COUNT	85500

#define S5P_SLEEP_CFG_OSC_EN	0

/* OTHERS Resgister */
#define S5P_OTHERS_USB_SIG_MASK 	(1 << 16)
#define S5P_OTHERS_MIPI_DPHY_EN		(1 << 28)

/* MIPI D-PHY Control Register 0 */
#define S5P_MIPI_PHY_CON0_M_RESETN	(1 << 1)
#define S5P_MIPI_PHY_CON0_S_RESETN	(1 << 0)

#endif /* CONFIG_CPU_S5PC100 */

#endif /* _PLAT_REGS_CLOCK_H */
