#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/gpio.h>
#include <linux/vgpio.h>
#include <linux/io.h>
#include <linux/interrupt.h>
#include <plat/irvine.h>
#include <asm/arch/bcm4760_reg.h>
#include <mach/hw_cfg.h>


static struct resource *card_detect;
static int card_in = 0;

static irqreturn_t irq_handler(int irq, void *dev_id)
{
	if(card_detect == NULL)
		return IRQ_NONE;

	if((readl(card_detect->start) & SDM1_F_CARDINSERTED_MASK) && ! card_in){
		card_in = 1;
		gpio_set_value(TT_VGPIO_SD_PWR_EN, 1);
	} 
	if (!(readl(card_detect->start) & SDM1_F_CARDINSERTED_MASK)){
		card_in = 0;
		gpio_set_value(TT_VGPIO_SD_PWR_EN, 0);
	}

	return IRQ_HANDLED;
}



static int sd_pwr_en_probe(struct platform_device *pdev)
{
	int ret = 0;
	struct resource	*card_int;

	card_detect = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	card_int = platform_get_resource(pdev, IORESOURCE_IO, 0);

	if(card_detect == NULL || card_int == NULL){
		printk(KERN_ERR"sd_pwr_en: Missing resource!\n");
		return -1;
	}

	ret = gpio_request(TT_VGPIO_SD_PWR_EN, "sd_pwr_en");
	if (ret){
		printk(KERN_ERR"sd_pwr_en: Can't get gpio: %d!\n",TT_VGPIO_SD_PWR_EN);
		return ret;
	}

	gpio_direction_output(TT_VGPIO_SD_PWR_EN, 0);

	if(readl(card_detect->start) & SDM1_F_CARDINSERTED_MASK){
		gpio_set_value(TT_VGPIO_SD_PWR_EN, 1);
	} else {
		gpio_set_value(TT_VGPIO_SD_PWR_EN, 0);
	}

	ret=request_irq(card_int->start, irq_handler, IRQF_SHARED, "sd_pwr_en", pdev);
	if (ret){
		printk(KERN_ERR"sd_pwr_en: Can't register interrupt!\n");
		return ret;
	}

	return ret;
}		

static int sd_pwr_en_suspend(struct platform_device *pdev)
{
	struct resource	*card_int;

	card_int = platform_get_resource(pdev, IORESOURCE_IO, 0);
	free_irq(card_int->start, pdev);
	gpio_set_value(TT_VGPIO_SD_PWR_EN, 0);
	return gpio_direction_input(TT_VGPIO_SD_PWR_EN);
}

static int sd_pwr_en_resume(struct platform_device *pdev)
{
	struct resource	*card_int;
	int ret = 0;

	card_int = platform_get_resource(pdev, IORESOURCE_IO, 0);
	ret=request_irq(card_int->start, irq_handler, IRQF_SHARED, "sd_pwr_en", pdev);
	if (ret){
		printk(KERN_ERR"sd_pwr_en: Can't register interrupt!\n");
		return ret;
	}
	gpio_direction_output(TT_VGPIO_SD_PWR_EN, 0);
	if(readl(card_detect->start) & SDM1_F_CARDINSERTED_MASK){
		card_in = 1;
		gpio_set_value(TT_VGPIO_SD_PWR_EN,1);
	} else {
		card_in = 0;
	}
	return ret;
}


static int sd_pwr_remove(struct platform_device *pdev)
{
	struct resource	*card_int;

	card_int = platform_get_resource(pdev, IORESOURCE_IO, 0);
	free_irq(card_int->start, pdev);
	gpio_free(TT_VGPIO_SD_PWR_EN);
	return 0;
}

static struct platform_driver sd_pwr_en_driver = {
	.probe 	 = sd_pwr_en_probe,
	.suspend = sd_pwr_en_suspend,
	.resume  = sd_pwr_en_resume,
	.remove	 = sd_pwr_remove,
	.driver  = {
		.owner	= THIS_MODULE,
		.name	= "sd_pwr_en",
	},
};

static int __init sd_pwr_en_init()
{
	int ret;

	ret = platform_driver_register(&sd_pwr_en_driver);
	if (ret)
		printk(KERN_ERR"Error creating the SD_PWR_EN platform driver\n");

	return ret;
}

arch_initcall(sd_pwr_en_init);
